/*
    测试题9:
        编写一个学生管理系统程序，用来记录学生的信息（包括姓名、年龄、性别、学号、分数），
        提供增加、删除、查询学生信息的入口。
*/

//引入头文件
#include <stdio.h>
#include <string.h>

//进行宏定义
#define N 100       //定义系统容量
#define NAME_LEN 22 //定义学生姓名长度
#define NUM_LEN 12  //定义学生学号长度

//***************定义部分*************
//定义学生信息结构体
typedef struct{
    //学生姓名
    char name[NAME_LEN];
    //学生年龄
    int age;
    //学生性别
    char sex[5];
    //学生学号
    char num[NUM_LEN];
    //学生分数
    float score;
    
}Student;

//存储所有学生的数组
Student stu[N]; //能够存储100个学生

//定义学生的个数
int totalStudent=0;

//接收用户输入的编号
int fno;

//定义数据文件保存的位置
char *filePath = "StudentInformation.data";

//*************函数声明部分************
int validateInput(int num,int min,int max);
void init();
void doAdd();
void doList();
void doDelete();
void doSearchByName();
void writeFile();


int main(int argc, const char * argv[]) {
    
    //系统初始化
    init();
    printf("系统初始化完成!\n");
    
    while (1) {
        
        //软件的界面显示出来
        printf("\n*************************************\n");
        printf("*********  欢迎使学生信息管理系统 *******\n");
        printf("*********  1、添加学生信息     *********\n");
        printf("*********  2、删除学生信息     *********\n");
        printf("*********  3、查看所有学生信息 **********\n");
        printf("*********  4、查询学生信息    **********\n");
        printf("*********  5、退出系统       ***********\n");
        printf("**************************************\n\n");
        
        //提示用户选择一个序号
        printf("请选择1-5之间的功能编号:\n");
        scanf("%d",&fno);
        //判断编号是否合法
        validateInput(fno, 1, 5);
        
        switch (fno) {
            case 1:
                //执行添加
                doAdd();
                break;
            case 2:
                //执行删除
                doDelete();
                break;
            case 3:
                //执行显示所有学生信息
                doList();
                break;
            case 4:
                //执行查找
                doSearchByName();
                break;
            case 5:
                //执行退出系统
                printf("系统正在退出...\n");
                printf("退出系统成功!\n");
                return 0;
                break;
                
            default:
                break;
        }
    
    }
    return 0;
}

/**
 *  验证输入是否合法
 *
 *  @param num 待验证的数
 *  @param min 最小值
 *  @param max 最大值
 *
 *  @return 1 非法，0正常
 */
int validateInput(int num,int min,int max){
    
    if (num<min||num>max) {
        printf("非法的输入\n");
        return 1;
    }
    return 0;
}

/**
    初始化工作
 
 */
void init(){
    
    //定义文件指针，并且打开文件
    FILE *fp = fopen(filePath, "r");
    
    if (fp!=NULL) {
        
        //读取学生信息
        fread(&totalStudent, sizeof(totalStudent), 1, fp);
        
        //读取每一个学生信息
        for (int i=0; i<totalStudent; i++) {
            //读取学生信息数据到数组中
            fread(&stu[i], sizeof(Student), 1, fp);
        }
    }else{
        
        //创建文件
        fp = fopen(filePath, "wb");
        //写入学生的个数
        fwrite(&totalStudent, sizeof(totalStudent), 1, fp);
        printf("学生信息系统文件创建成功!\n");
        
    }
    //关闭文件指针
    fclose(fp);
}

/**
 *  添加学生信息
 */
void doAdd(){
    
    printf("您选择的是添加学生信息,请按照提示操作:\n");
    
    //1、提示用户输入学生姓名，接收学生姓名
    printf("请输入学生姓名(注意不要有空格):\n");
    scanf("%s",stu[totalStudent].name);
    
    //2、提示用户输入学生年龄，接收学生年龄
    printf("请输入学生年龄:\n");
    scanf("%d",&stu[totalStudent].age);
    
    //3、提示用户输入学生性别，接收学生性别
    printf("请输入学生性别(m.男 f.女):\n");
    scanf("%s",stu[totalStudent].sex);
    
    //4、提示用户输入学生学号，接收学生学号
    printf("请输入学生学号(注意不要有空格):\n");
    scanf("%s",stu[totalStudent].num);
    
    //5、提示用户输入学生分数，接收学生分数
    printf("请输入学生分数:\n");
    scanf("%f",&stu[totalStudent].score);
    
    //确认操作
    int flag;
    printf("您确认要添加吗?\n1.确认 0.取消\n");
    scanf("%d",&flag);
    if (flag) {
        printf("添加成功!\n");
        //学生个数加1
        totalStudent++;
        //把学生信息写入到文件中
        writeFile();
    }
}
/**
 *  显示所有学生
 */
void doList(){
    
    printf("您选择的是显示所有学生，学生列表如下:\n");
    
    //判断学生信息是否为空
    if (totalStudent==0) {
        printf("您的信息库为空，请添加学生信息!\n");
    }else{
        //输出格式
        printf("编号\t 姓名\t 年龄\t 性别\t 学号\t 分数\t\n");
        //遍历学生信息数组
        for (int i=0; i<totalStudent; i++) {
            
            printf("%d\t %s\t %d\t\t %s\t\t %s\t %.2f\t\n",i+1,stu[i].name,stu[i].age,stu[i].sex,stu[i].num,stu[i].score);
            
        }
        
    }
    
}
/**
 *  删除学生信息
 */
void doDelete(){
    
    printf("您选择的是删除学生，请按照提示操作:\n");
    
    //1、判断是否为空
    if (totalStudent==0) {
        printf("您的信息为空，请添加学生信息!\n");
    }else{
        //输出格式
        printf("编号\t 姓名\t 年龄\t 性别\t 学号\t 分数\t\n");
        //遍历学生信息数组
        for (int i=0; i<totalStudent; i++) {
            
            printf("%d\t %s\t %d\t\t %s\t\t %s\t %.2f\t\n",i+1,stu[i].name,stu[i].age,stu[i].sex,stu[i].num,stu[i].score);
            
        }
        int num;
        //4、提示选择您要删除的编号
        printf("输入您要删除的学生编号:\n");
        scanf("%d",&num);
        //判断是否非法
        if(validateInput(num, 1, totalStudent)){
            
            return ;
        }
        //5、确认操作
        int flag;
        printf("您确认要删除%d\t%s 吗?\n1.确认 0.取消\n",num,stu[num-1].name);
        scanf("%d",&flag);
        if (flag) {
            for (int i=num-1; i<totalStudent-1; i++) {
                //将数组数据从编号位置前移覆盖
                strcpy(stu[i].name, stu[i+1].name);
                stu[i].age=stu[i+1].age;
                strcpy(stu[i].sex, stu[i+1].sex);
                strcpy(stu[i].num, stu[i+1].num);
                stu[i].score=stu[i+1].score;
            }
            printf("删除成功!\n");
            //个数减1
            totalStudent--;
            //把信息写入到文件中
            writeFile();
            
        }
    }
}
/**
 *  查询学生
 */
void doSearchByName(){
    
    printf("您选择的是查询学生信息，请按照提示操作:\n");
    
    //判断是否为空
    if (totalStudent==0) {
        printf("您的信息库为空，请添加学生信息!\n");
    }else{
        Student arr; //新建一个结构变量来保存用户搜索数据
        //提示用户输入要搜索的姓名或学号
        printf("请您输入要搜索的姓名或者学号(注意不要有空格)\n");
        scanf("%s",arr.name);
        int n=0;    //定义一个搜索计数
        //与数组内数据进行对比
        for (int i=0; i<totalStudent; i++) {
            //如果一致则显示搜索结果
            if (strcmp(arr.name,stu[i].name)==0||strcmp(arr.name,stu[i].num)==0) {
                n++;
                printf("搜索结果如下:\n");
                printf("编号\t 姓名\t 年龄\t 性别\t 学号\t 分数\t\n");
                printf("%d\t %s\t %d\t\t %s\t\t %s\t %.2f\t\n",i+1,stu[i].name,stu[i].age,stu[i].sex,stu[i].num,stu[i].score);
            }
            
        }
        //如果计数n=0则是未找到，则显示没有该学生信息
        if (n==0) {
            printf("没有该学生信息\n");
        }
    }
}

/**
 *  写入文件中
 */
void writeFile(){
    
    //1、以wb方式打开文件
    FILE *fp = fopen(filePath, "wb");
    if (fp!=NULL) {
        
        //先写学生个数
        fwrite(&totalStudent, sizeof(totalStudent), 1, fp);
        
        //写每个学生数据
        for (int i=0; i<totalStudent; i++) {
            fwrite(&stu[i], sizeof(Student), 1, fp);
            
        }
        
        printf("文件更新成功\n");
    }
    
    //关闭文件
    fclose(fp);
    
}